package com.tencent.qcloud.tuikit.tuiconversation.classicui.widget;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.util.Log;
import android.widget.ImageView;
import android.widget.RelativeLayout;

import com.tencent.qcloud.tuicore.TUIConfig;
import com.tencent.qcloud.tuikit.timcommon.component.gatherimage.SynthesizedImageView;
import com.tencent.qcloud.tuikit.timcommon.component.interfaces.IUIKitCallback;
import com.tencent.qcloud.tuikit.timcommon.util.ImageUtil;
import com.tencent.qcloud.tuikit.timcommon.util.ScreenUtil;
import com.tencent.qcloud.tuikit.timcommon.util.ThreadUtils;
import com.tencent.qcloud.tuikit.tuiconversation.R;
import com.tencent.qcloud.tuikit.tuiconversation.bean.ConversationInfo;
import com.tencent.qcloud.tuikit.tuiconversation.presenter.ConversationIconPresenter;

import java.util.ArrayList;
import java.util.List;

/**
 * 会话列表头像View
 */
public class ConversationIconView extends RelativeLayout {

    private static final int icon_size = ScreenUtil.getPxByDp(50);
    private ImageView mIconView;
    private boolean showFoldedStyle = false;

    private ConversationIconPresenter presenter;

    public ConversationIconView(Context context) {
        super(context);
        init();
    }

    public ConversationIconView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public ConversationIconView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {
        inflate(getContext(), com.tencent.qcloud.tuikit.timcommon.R.layout.profile_icon_view, this);
        mIconView = findViewById(com.tencent.qcloud.tuikit.timcommon.R.id.profile_icon);
        ((SynthesizedImageView) mIconView).defaultImage(TUIConfig.getDefaultAvatarImage());
        presenter = new ConversationIconPresenter();
    }

    public void setShowFoldedStyle(boolean showFoldedStyle) {
        this.showFoldedStyle = showFoldedStyle;
    }

    /**
     * 设置会话头像的url
     *
     * @param iconUrls 头像url,最多只取前9个
     */
    public void setIconUrls(final List<Object> iconUrls, final String conversationId) {
        // 需要在主线程中执行，以免写缓存出现问题
        ThreadUtils.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (mIconView instanceof SynthesizedImageView) {
                    ((SynthesizedImageView) (mIconView)).displayImage(iconUrls).load(conversationId);
                }
            }
        });
    }

    public void setConversation(ConversationInfo conversationInfo, int position) {

        if (conversationInfo.isSystem()) {
            Log.e("simonImage", "position:" + position + ":1111111111111111111");
            mIconView.setImageResource(R.drawable.ic_sys_msg);
        } else if (showFoldedStyle && conversationInfo.isMarkFold()) {
            Log.e("simonImage", "position:" + position + ":222222222222222222222");
            mIconView.setImageResource(R.drawable.ic_fold);
        } else if (mIconView instanceof SynthesizedImageView) {
            ((SynthesizedImageView) (mIconView)).setImageId(conversationInfo.getConversationId());
            if (conversationInfo.isGroup()) {
                Log.e("simonImage", "position:" + position + ":444444444444444444");
                fillConversationUrlForGroup(conversationInfo);
            } else {
                Log.e("simonImage", "position:" + position + ":55555555555555555");
                setIconUrls(conversationInfo.getIconUrlList(), conversationInfo.getConversationId());
            }
        } else {
            Log.e("simonImage", "position:" + position + ":少年出大事了");
        }
    }

    private void fillConversationUrlForGroup(final ConversationInfo info) {
        List<Object> iconUrlList = info.getIconUrlList();
        if (iconUrlList == null || iconUrlList.size() == 0) {
            if (!TUIConfig.isEnableGroupGridAvatar()) {
                List<Object> faceList = new ArrayList<>();
                faceList.add(TUIConfig.getDefaultGroupAvatarImage());
                setIconUrls(faceList, info.getConversationId());
                return;
            }
            // 读取文件，在线程池中进行，避免主线程卡顿
            ThreadUtils.execute(new Runnable() {
                @Override
                public void run() {
                    final String savedIcon = ImageUtil.getGroupConversationAvatar(info.getConversationId());
                    if (TextUtils.isEmpty(savedIcon)) {
                        fillFaceUrlList(info.getId(), info);
                    } else {
                        List<Object> list = new ArrayList<>();
                        list.add(savedIcon);
                        info.setIconUrlList(list);
                        setIconUrls(list, info.getConversationId());
                    }
                }
            });
        } else {
            if (!TUIConfig.isEnableGroupGridAvatar() && iconUrlList.size() > 1) {
                List<Object> faceList = new ArrayList<>();
                faceList.add(TUIConfig.getDefaultGroupAvatarImage());
                setIconUrls(faceList, info.getConversationId());
            } else {
                setIconUrls(iconUrlList, info.getConversationId());
            }
        }
    }

    private void fillFaceUrlList(final String groupID, final ConversationInfo info) {
        ThreadUtils.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                clearImage();
                presenter.getGroupMemberIconList(groupID, new IUIKitCallback<List<Object>>() {
                    @Override
                    public void onSuccess(List<Object> data) {
                        info.setIconUrlList(data);
                        setIconUrls(data, info.getConversationId());
                    }

                    @Override
                    public void onError(String module, int errCode, String errMsg) {

                    }
                });
            }
        });
    }

    public void setRadius(int radius) {
        if (mIconView instanceof SynthesizedImageView) {
            ((SynthesizedImageView) (mIconView)).setRadius(radius);
        }
    }

    public void setDefaultImageResId(int resId) {
        BitmapDrawable bd = (BitmapDrawable) getContext().getResources().getDrawable(resId);
        mIconView.setImageBitmap(bd.getBitmap());
    }

    public void setBitmapResId(int resId) {
        BitmapDrawable bd = (BitmapDrawable) getContext().getResources().getDrawable(resId);
        Bitmap bitmap = ImageUtil.toRoundBitmap(bd.getBitmap());
        mIconView.setImageBitmap(bitmap);
    }

    public void clearImage() {
        if (mIconView instanceof SynthesizedImageView) {
            ((SynthesizedImageView) mIconView).clear();
        }
    }
}

